﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EFEncrypt
{
    public class Context : DbContext
    {
        public Context()
            : base("name=context")
        {

        }
        public DbSet<Person> People { get; set; }

        public override int SaveChanges()
        {
            Encrypt<Person>();
            return base.SaveChanges();
        }

        public override Task<int> SaveChangesAsync(System.Threading.CancellationToken cancellationToken)
        {
            Encrypt<Person>();
            return base.SaveChangesAsync(cancellationToken);
        }

        public void Encrypt<T>()
            where T : class
        {
            Stopwatch timer = new Stopwatch();
            timer.Start();
            var changedPeople = this.ChangeTracker.Entries<T>()
                .Where(one => one.State != EntityState.Detached && one.State != EntityState.Unchanged).ToList();
            foreach (var item in changedPeople)
            {
                foreach (var propertyName in item.CurrentValues.PropertyNames)
                {
                    var value = item.CurrentValues[propertyName];
                    var stringValue = value as string;
                    if (stringValue != null)
                    {
                        if (!string.IsNullOrEmpty(stringValue))
                        {
                            item.CurrentValues[propertyName] = EncryptOneString(stringValue);
                        }
                    }
                }
            }
            timer.Stop();
            Console.WriteLine("Total time to encrypt one person is {0} ms", timer.ElapsedMilliseconds);
        }

        public void Decrypt<T>(IEnumerable<T> collection)
            where T: class
        {
            foreach (var item in collection)
            {
                var entry = Entry(item);
                DecryptEntity(entry);
            }
        }


        private void DecryptEntity(DbEntityEntry item)
        {
            foreach (var propertyName in item.CurrentValues.PropertyNames)
            {
                var value = item.CurrentValues[propertyName];
                var stringValue = value as string;
                if (stringValue != null)
                {
                    if (!string.IsNullOrEmpty(stringValue))
                    {
                        item.CurrentValues[propertyName] = DecryptOneString(stringValue);
                    }
                }
            }
        }

        public string DecryptOneString(string input)
        {
            return EncryptionUtility.Decrypt(input, "MyRandomPassword");
        }

        public string EncryptOneString(string input)
        {
            return EncryptionUtility.Encrypt(input, "MyRandomPassword");
        }
    }
}
